try:

    from dodfminer.extract.polished import acts
    from dodfminer.downloader.core import Downloader
    from dodfminer.extract.polished.core import ActsExtractor
    from dodfminer.extract.pure.core import ContentExtractor
except:
    pass